from o3seespy.base_model import OpenSeesObject


class TransformationBase(OpenSeesObject):
    op_base_type = "geomTransf"


class Linear(TransformationBase):
    op_type = 'Linear'

    def __init__(self, osi, vecxz, d_i=None, d_j=None):
        raise ValueError('Use geom_transf.Linear not transformation.Linear')
        self.vecxz = vecxz
        self.d_i = d_i
        self.d_j = d_j

        osi.n_transformation += 1
        self._tag = osi.n_transformation
        self._parameters = [self.op_type, self._tag, *self.vecxz]
        if d_i is not None:
            self._parameters += ['jntOffset', self.d_i, self.d_j]
        self.to_process(osi)




import openseespy.opensees as opy
import pytest


def test_elastic_isotropic():
    opy.wipe()
    opy.model('basic', '-ndm', 2, '-ndf', 3)
    v_is_int = 1
    v_is_float = 1.
    # with pytest.raises(opy.error):  # TODO: can't find error class
    #     opy.nDMaterial('ElasticIsotropic', 1, 1., v_is_int, 0.0)
    opy.nDMaterial('ElasticIsotropic', 1, 1., v_is_float, 0.0)

if __name__ == '__main__':
    test_elastic_isotropic()


from inspect import signature
from collections import OrderedDict


def to_commands(op_base_type, parameters):
    para = []
    for i, e in enumerate(parameters):
        if isinstance(e, str):
            e = "'%s'" % e
        elif isinstance(e, float):
            e = '%.6g' % e
            if '.' not in e and 'e' not in e:
                e += '.0'
        para.append(str(e))
        if i > 40:  # avoid verbose print output
            break
    p_str = ', '.join(para)
    return 'opy.%s(%s)' % (op_base_type, p_str)


def to_py_file(osi, ofile='ofile.py', w_analyze=False):
    ofile = open(ofile, 'w')
    pstr = 'import openseespy.opensees as opy\n' + '\n'.join(osi.commands)
    if w_analyze:
        pstr += '\nopy.analyze(1, 0.1)\n'
    ofile.write(pstr)
    ofile.close()


def get_o3_kwargs_from_obj(obj, o3_obj, custom=None, overrides=None):
    if custom is None:
        custom = {}
    if overrides is None:
        overrides = {}
    sig = signature(o3_obj)
    kwargs = OrderedDict()
    args = []
    sig_vals = sig.parameters.values()
    for p in sig_vals:
        if p.name in custom:
            pname = custom[p.name]
        else:
            pname = p.name
        if pname == 'osi':
            continue
        if pname in overrides:
            val = overrides[pname]
        else:
            try:
                val = getattr(obj, pname)
            except AttributeError as e:
                if p.default == p.empty:
                    raise AttributeError(e)
                else:
                    val = p.default
        if p.default == p.empty:
            args.append(val)
        else:
            if val is not None:
                kwargs[p.name] = val
    return args, kwargs


def has_o3_model_changed(cur_type, prev_type, cur_args, prev_args, cur_kwargs, prev_kwargs):
    import numpy as np
    changed = 0
    if cur_type != prev_type or len(cur_args) != len(prev_args) or len(cur_kwargs) != len(prev_kwargs):
        changed = 1
    else:
        for j, arg in enumerate(cur_args):
            if hasattr(arg, '__len__'):
                if len(arg) != len(prev_args[j]):
                    changed = 1
                    break
                for k, subarg in enumerate(arg):
                    if not np.isclose(subarg, prev_args[j][k]):
                        changed = 1
            elif not np.isclose(arg, prev_args[j]):
                changed = 1
                break
        for pm in cur_kwargs:
            if pm not in prev_kwargs:
                changed = 1
                break
            if hasattr(cur_kwargs[pm], '__len__'):
                if len(cur_kwargs[pm]) != len(prev_kwargs[pm]):
                    changed = 1
                    break
                for k, subarg in enumerate(cur_kwargs[pm]):
                    if not np.isclose(subarg, cur_kwargs[pm][k]):
                        changed = 1
                        break
            elif not np.isclose(cur_kwargs[pm], prev_kwargs[pm]):
                changed = 1
                break
    return changed


import o3seespy as o3  # for testing only


def test_initial_state_analysis_wrapper():
    osi = o3.OpenSeesInstance(ndm=2)
    mat = o3.nd_material.ElasticIsotropic(osi, e_mod=1.0, nu=1.0, rho=0.0)
    o3.nd_material.InitialStateAnalysisWrapper(osi, n_d_mat=mat, n_dim=1)




from o3seespy.base_model import OpenSeesObject


class Node(OpenSeesObject):
    op_base_type = "node"
    op_type = "node"
    # x_con = None
    # y_con = None
    # z_con = None
    # x_rot_con = None
    # y_rot_con = None
    # z_rot_con = None
    # x_mass = 0.0
    # y_mass = 0.0
    # z_rot_mass = 0.0

    def __init__(self, osi, x: float, y=None, z=None, vel=None, acc=None,
                 x_mass=None, y_mass=None, z_mass=None, x_rot_mass=None, y_rot_mass=None, z_rot_mass=None):
        """
        An OpenSEES node

        Parameters
        ----------
        osi : opensees_pack.opensees_instance.OpenSeesInstance object
            An instance of opensees
        x : float
            x-coordinate
        y : float, optional
            y-coordinate
        z : float, optional
            z-coordinate
        vel : iterable object, optional
            nodal velocities (x, y, z)
        acc : iterable object, optional
        """

        self.x = float(x)
        if y is not None:
            self.y = float(y)
        if z is not None:
            self.z = float(z)
        self.x_mass = x_mass
        self.y_mass = y_mass
        self.z_mass = z_mass
        self.x_rot_mass = x_rot_mass
        self.y_rot_mass = y_rot_mass
        self.z_rot_mass = z_rot_mass
        self.vel = vel
        self.acc = acc
        osi.n_node += 1
        self._tag = osi.n_node
        if osi.ndm == 1:
            self._parameters = [self._tag, *[self.x]]
            pms = ['x_mass']
        elif osi.ndm == 2:
            self._parameters = [self._tag, *[self.x, self.y]]
            pms = ['x_mass', 'y_mass', 'z_rot_mass']
        elif osi.ndm == 3:
            self._parameters = [self._tag, *[self.x, self.y, self.z]]
            pms = ['x_mass', 'y_mass', 'z_mass', 'x_rot_mass', 'y_rot_mass', 'z_rot_mass']
        else:
            raise NotImplementedError("Currently only supports 1-3D analyses")
        masses = []
        none_found = 0
        for pm in pms:
            val = getattr(self, pm)
            if val is None:
                none_found = pm
            else:
                setattr(self, pm, float(val))
                if not none_found:
                    masses.append(float(val))
                else:
                    raise ValueError(f'Cannot set {pm} since {none_found} is None')
        if len(masses):
            self._parameters += ["-mass", *[self.x_mass]]
        if self.vel is not None:
            self._parameters += ["-vel", self.vel]
        if self.acc is not None:
            self._parameters += ["-accel", self.acc]
        self.to_process(osi)



import o3seespy as o3  # for testing only
import pytest

def test_std_brick():
    osi = o3.OpenSeesInstance(ndm=3)
    coords = [[0, 0, 0], [1, 0, 0], [1, 1, 0], [0, 1, 0], [0, 0, 1], [1, 0, 1], [1, 1, 1], [0, 1, 1]]
    ele_nodes = [o3.node.Node(osi, *coords[x]) for x in range(len(coords))]
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.StdBrick(osi, ele_nodes=ele_nodes, mat=mat, b1=1.0, b2=1.0, b3=1.0)


def test_bbar_brick():
    osi = o3.OpenSeesInstance(ndm=3)
    coords = [[0, 0, 0], [1, 0, 0], [1, 1, 0], [0, 1, 0], [0, 0, 1], [1, 0, 1], [1, 1, 1], [0, 1, 1]]
    ele_nodes = [o3.node.Node(osi, *coords[x]) for x in range(len(coords))]
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.BbarBrick(osi, ele_nodes=ele_nodes, mat=mat, b1=1.0, b2=1.0, b3=1.0)


@pytest.mark.skip()
def test_brick20n():
    osi = o3.OpenSeesInstance(ndm=3)
    coords = [[0, 0, 0], [1, 0, 0], [1, 1, 0], [0, 1, 0], [0, 0, 1], [1, 0, 1], [1, 1, 1], [0, 1, 1]]
    ele_nodes = [o3.node.Node(osi, *coords[x]) for x in range(len(coords))]
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.Brick20N(osi, ele_nodes=ele_nodes, mat=mat, bf1=1.0, bf2=1.0, bf3=1.0, mass_den=1.0)


def test_ss_pbrick():
    osi = o3.OpenSeesInstance(ndm=3)
    coords = [[0, 0, 0], [1, 0, 0], [1, 1, 0], [0, 1, 0], [0, 0, 1], [1, 0, 1], [1, 1, 1], [0, 1, 1]]
    ele_nodes = [o3.node.Node(osi, *coords[x]) for x in range(len(coords))]
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.SSPbrick(osi, ele_nodes=ele_nodes, mat=mat, b1=1.0, b2=1.0, b3=1.0)




from o3seespy.base_model import OpenSeesObject


class AlgorithmBase(OpenSeesObject):
    op_base_type = "algorithm"


class Linear(AlgorithmBase):
    op_type = "Linear"

    def __init__(self, osi, secant=False, initial=False, factor_once=False):
        self.secant = secant
        self.initial = initial
        self.factor_once = factor_once
        self._parameters = [self.op_type, self.secant, self.initial, self.factor_once]
        self.to_process(osi)


class Newton(AlgorithmBase):
    op_type = "Newton"

    def __init__(self, osi, secant=False, initial=False, initial_then_current=False):
        self.secant = secant
        self.initial = initial
        self.initial_then_current = initial_then_current
        self._parameters = [self.op_type, self.secant, self.initial, self.initial_then_current]
        self.to_process(osi)



from o3seespy.opensees_instance import OpenSeesInstance
from o3seespy import exceptions
from o3seespy import cc
from o3seespy import cc as static  # deprecated
from o3seespy.command import node, algorithm, rayleigh, test, uniaxial_material, element, nd_material
from o3seespy.command.common import *
from o3seespy.command import section, beam_integration, transformation, constraints, numberer, system, region
from o3seespy.command import integrator, analysis, recorder, pattern, time_series, geom_transf, patch, layer
import o3seespy.tools
from o3seespy.command import test_check  # deprecated
from o3seespy.__about__ import __version__




from tests.binary import functions as fn
import openseespy.opensees as opy


def test_force_beam_column():

    opy.wipe()
    opy.model('basic', '-ndm', 2, '-ndf', 3)  # 2 dimensions, 3 dof per node
    integ_tag = 1
    ele_tag = 1
    fn.define_beam_integration(integ_tag=integ_tag)
    fn.define_force_beam_column(ele_tag, integ_tag)



import o3seespy as o3  # for testing only
import pytest


@pytest.mark.skip()
def test_simple_contact2d():
    osi = o3.OpenSeesInstance(ndm=2)
    i_node = o3.node.Node(osi, 0.0, 0.0)
    j_node = o3.node.Node(osi, 0.0, 1.0)
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.SimpleContact2D(osi, i_node=i_node, j_node=j_node, s_node=1, l_node=1, mat=mat, g_tol=1.0, f_tol=1.0)


@pytest.mark.skip()
def test_simple_contact3d():
    osi = o3.OpenSeesInstance(ndm=2)
    i_node = o3.node.Node(osi, 0.0, 0.0)
    j_node = o3.node.Node(osi, 0.0, 1.0)
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.SimpleContact3D(osi, i_node=i_node, j_node=j_node, k_node=1, l_node=1, s_node=1, lagr_node=1, mat=mat, g_tol=1.0, f_tol=1.0)


@pytest.mark.skip()
def test_beam_contact2d():
    osi = o3.OpenSeesInstance(ndm=2)
    i_node = o3.node.Node(osi, 0.0, 0.0)
    j_node = o3.node.Node(osi, 0.0, 1.0)
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.BeamContact2D(osi, i_node=i_node, j_node=j_node, s_node=1, l_node=1, mat=mat, width=1.0, g_tol=1.0, f_tol=1.0, c_flag=1)


@pytest.mark.skip()
def test_beam_contact3d():
    osi = o3.OpenSeesInstance(ndm=2)
    i_node = o3.node.Node(osi, 0.0, 0.0)
    j_node = o3.node.Node(osi, 0.0, 1.0)
    mat = o3.nd_material.ElasticIsotropic(osi, 1, 0.45)
    o3.element.BeamContact3D(osi, i_node=i_node, j_node=j_node, s_node=1, l_node=1, radius=1.0, crd_transf=1, mat=mat, g_tol=1.0, f_tol=1.0, c_flag=1)


@pytest.mark.skip()
def test_beam_end_contact3d():
    osi = o3.OpenSeesInstance(ndm=2)
    i_node = o3.node.Node(osi, 0.0, 0.0)
    j_node = o3.node.Node(osi, 0.0, 1.0)
    o3.element.BeamEndContact3D(osi, i_node=i_node, j_node=j_node, s_node=1, l_node=1, radius=1.0, g_tol=1.0, f_tol=1.0, c_flag=1.0)




FREE = 0
FIXED = 1

X = 1
Y = 2
ROTZ = 3
DOF2D_X = 1
DOF2D_Y = 2
DOF2D_ROTZ = 3

P = 'P'
M_Z = 'Mz'
V_Y = 'Vy'
M_Y = 'My'
V_Z = 'Vz'
TORSION = 'T'

PLANE_STRAIN = "PlaneStrain"
PLANE_STRESS = "PlaneStress"




from o3seespy.base_model import OpenSeesObject


class SystemBase(OpenSeesObject):
    op_base_type = "system"


class ProfileSPD(SystemBase):
    op_type = "ProfileSPD"

    def __init__(self, osi):

        self._parameters = [self.op_type]
        self.to_process(osi)


class SparseGeneral(SystemBase):
    op_type = "SparseGeneral"

    def __init__(self, osi):

        self._parameters = [self.op_type]
        self.to_process(osi)


class BandGeneral(SystemBase):
    op_type = "BandGeneral"

    def __init__(self, osi):

        self._parameters = [self.op_type]
        self.to_process(osi)


class FullGeneral(SystemBase):
    op_type = "FullGeneral"

    def __init__(self, osi):
        self._parameters = [self.op_type]
        self.to_process(osi)